/*
 * Exotic - C/C++ source code testing
 * Copyright (c) 2007, Jan Vidar Krey
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static void exotic_version();

#ifndef __EXOTIC__STANDALONE__
#include "autotest.h"
static void exotic_version()
{
	printf("Extatic.org Test Infrastructure: exotic " VERSION "\n\n");
	printf("Copyright (C) 2007 Jan Vidar Krey, janvidar@extatic.org\n");
	printf("This is free software with ABSOLUTELY NO WARRANTY.\n\n");
}
#endif

struct exo_test_data
{
	exo_test_t test;
	char* name;
	struct exo_test_data* next;
};


static void exotic_summary(struct exotic_handle* handle)
{
	int total     = handle->pass + handle->fail;
	int pass_rate = total ? (100*handle->pass / total) : 0;
	int fail_rate = total ? (100*handle->fail / total) : 0;
	
	printf("\n");
	printf("--------------------------------------------\n");
	printf("Results:\n");
	printf(" * Total tests run:  %8d\n", total);
	printf(" * Tests passed:     %8d (~%d%%)\n", (int) handle->pass, pass_rate);
	printf(" * Tests failed:     %8d (~%d%%)\n", (int) handle->fail, fail_rate);
	printf("--------------------------------------------\n");
}


static void exotic_help(const char* program)
{
	printf("Usage: %s [OPTIONS]\n\n", program);
	printf("Options:\n");
	printf("  --help      -h    Show this message\n");
	printf("  --version   -v    Show version\n");
	printf("  --summary   -s    show only summary)\n");
	printf("  --fail      -f    show only test failures\n");
	printf("  --pass      -p    show only test passes\n");
	printf("\nExamples:\n");
	printf("  %s -s -f\n\n", program);
}


int exotic_initialize(struct exotic_handle* handle, int argc, char** argv)
{
	int n;
	if (!handle || !argv) return -1;
	
	memset(handle, 0, sizeof(struct exotic_handle));
	
	for (n = 1; n < argc; n++)
	{
		if (!strcmp(argv[n], "-h") || !strcmp(argv[n], "--help"))
		{
			exotic_help(argv[0]);
			exit(0);
		}
		
		if (!strcmp(argv[n], "-v") || !strcmp(argv[n], "--version"))
		{
			exotic_version();
			exit(0);
		}
		
		if (!strcmp(argv[n], "-s") || !strcmp(argv[n], "--summary"))
		{
			handle->config_show_summary = cfg_on;
			continue;
		}
		
		if (!strcmp(argv[n], "-f") || !strcmp(argv[n], "--fail"))
		{
			handle->config_show_fail = cfg_on;
			continue;
		}
		
		if (!strcmp(argv[n], "-p") || !strcmp(argv[n], "--pass"))
		{
			handle->config_show_pass = cfg_on;
			continue;
		}
		
		fprintf(stderr, "Unknown argument: %s\n\n", argv[n]);
		exotic_help(argv[0]);
		exit(0);
	}
	
	if (handle->config_show_summary == cfg_on)
	{
		if (handle->config_show_pass == cfg_default) handle->config_show_pass = cfg_off;
		if (handle->config_show_fail == cfg_default) handle->config_show_fail = cfg_off;
	
	}
	else if (handle->config_show_pass == cfg_on)
	{
		if (handle->config_show_summary == cfg_default) handle->config_show_summary = cfg_off;
		if (handle->config_show_fail    == cfg_default) handle->config_show_fail = cfg_off;
	}
	else if (handle->config_show_fail == cfg_on)
	{
		if (handle->config_show_summary == cfg_default) handle->config_show_summary = cfg_off;
		if (handle->config_show_fail    == cfg_default) handle->config_show_fail = cfg_off;
	}
	else
	{
		if (handle->config_show_summary == cfg_default) handle->config_show_summary = cfg_on;
		if (handle->config_show_fail    == cfg_default) handle->config_show_fail = cfg_on;
		if (handle->config_show_pass    == cfg_default) handle->config_show_pass = cfg_on;
	}
	return 0;
}


void exotic_add_test(struct exotic_handle* handle, exo_test_t func, const char* name)
{
	if (!handle)
	{
		fprintf(stderr, "exotic_add_test: failed, no handle!\n");
		exit(-1);
	}
	
	struct exo_test_data* test = (struct exo_test_data*) malloc(sizeof(struct exo_test_data));
	if (!test)
	{
		fprintf(stderr, "exotic_add_test: out of memory!\n");
		exit(-1);
	}
	
	/* Create the test */
	memset(test, 0, sizeof(struct exo_test_data));
	test->test = func;
	test->name = strdup(name);
	
	/* Register the test in the handle */
	if (!handle->first)
	{
		handle->first = test;
		handle->current = test;
	}
	else
	{
		handle->current->next = test;
		handle->current = test;
	}
}


int exotic_run(struct exotic_handle* handle)
{
	struct exo_test_data* tmp = NULL;
	
	if (!handle)
	{
		fprintf(stderr, "Error: exotic is not initialized\n");
		return -1;
	}

	handle->current = handle->first;
	while (handle->current)
	{
		tmp = handle->current;
		
		if (handle->current->test()) {
			if (handle->config_show_pass == cfg_on) printf("* PASS test '%s'\n", tmp->name);
			handle->pass++;
		} else {
			if (handle->config_show_fail == cfg_on) printf("* FAIL test '%s'\n", tmp->name);
			handle->fail++;
		}
		
		handle->current = handle->current->next;
		
		free(tmp->name);
		free(tmp);
	}
	
	if (!handle->first)
	{
		printf("(No tests added)\n");
	}
	
	if (handle->config_show_summary == cfg_on)
		exotic_summary(handle);

	return (handle->fail) ? handle->fail : 0;
}

